#include <QApplication>
#include <QGraphicsView>
#include <QGraphicsScene>
#include <QGraphicsPixmapItem>
#include <QPropertyAnimation>
#include <QSequentialAnimationGroup>
#include <QParallelAnimationGroup>
#include <QLayout>
#include <QPushButton>

class MovableGraphicsPixmapItem : public QObject, public QGraphicsPixmapItem
{
    Q_OBJECT
    Q_PROPERTY( QPointF pos READ pos WRITE setPos )

public:
    MovableGraphicsPixmapItem( const QPixmap& pixmap, QGraphicsItem* parent = 0 )
	: QObject(), QGraphicsPixmapItem( pixmap, parent ) {
    }
};

class HarnessPrivate
{
public:
    HarnessPrivate() 
	: movingAreaSize( QSize( 360, 320 ) ), horizontalOffset( 8 ) {
    }

    QSize movingAreaSize;
    int horizontalOffset;
    QParallelAnimationGroup* ballAnimationGroup;
};

class Harness : public QWidget
{
    Q_OBJECT
    Q_DECLARE_PRIVATE( Harness )

public:
    Harness();
    ~Harness();

private slots:
    void startAnimation();

private:
    MovableGraphicsPixmapItem* createBallItem( const QString& pixmapFileName );
    QPropertyAnimation* createBallAnimation( MovableGraphicsPixmapItem* item, int duration, int startX, int endX, int y );
    HarnessPrivate* d_ptr;
};

Harness::Harness()
    : QWidget( 0 ), d_ptr( new HarnessPrivate )
{
    Q_D( Harness );

    QGraphicsScene* graphicsScene = new QGraphicsScene( 0, 0, d->movingAreaSize.width(), d->movingAreaSize.height() );
    MovableGraphicsPixmapItem* yellowBallItem = createBallItem( ":/images/YellowGlassBall.png" );
    QSize ballSize = yellowBallItem->pixmap().size();
    int ballSpacingY = ( d->movingAreaSize.height() - ballSize.height() * 3 ) / 4;
    int yellowBallY = ballSpacingY;
    yellowBallItem->setPos( d->horizontalOffset, yellowBallY );
    graphicsScene->addItem( yellowBallItem );

    MovableGraphicsPixmapItem* greenBallItem = createBallItem( ":/images/GreenGlassBall.png" );
    int greenBallY = yellowBallY + ballSize.height() + ballSpacingY;
    greenBallItem->setPos( d->horizontalOffset, greenBallY );
    graphicsScene->addItem( greenBallItem );

    MovableGraphicsPixmapItem* redBallItem = createBallItem( ":/images/RedGlassBall.png" );
    int redBallY = greenBallY + ballSize.height() + ballSpacingY;
    redBallItem->setPos( d->horizontalOffset, redBallY );
    graphicsScene->addItem( redBallItem );

    QGraphicsView* graphicsView = new QGraphicsView();
    graphicsView->setScene( graphicsScene );

    QPushButton* animateButton = new QPushButton( "Animate" );

    QHBoxLayout* buttonLayout = new QHBoxLayout;
    buttonLayout->addStretch();
    buttonLayout->addWidget( animateButton );

    QVBoxLayout* topLayout = new QVBoxLayout;
    topLayout->addWidget( graphicsView );
    topLayout->addLayout( buttonLayout );

    setLayout( topLayout );
    setFixedSize( sizeHint() );

    int movableBallStartX = d->horizontalOffset;
    int movableBallEndX = d->movingAreaSize.width() - ballSize.width() - d->horizontalOffset;

    QPropertyAnimation* yellowBallAnimation = createBallAnimation( yellowBallItem, 20 * 1000, movableBallStartX, movableBallEndX, yellowBallY );
    QPropertyAnimation* greenBallAnimation = createBallAnimation( greenBallItem, 10 * 1000, movableBallStartX, movableBallEndX, greenBallY );
    QPropertyAnimation* redBallAnimation = createBallAnimation( redBallItem, 10 * 1000, movableBallStartX, movableBallEndX, redBallY );

    d->ballAnimationGroup = new QParallelAnimationGroup( this );
    d->ballAnimationGroup->addAnimation( yellowBallAnimation );
    QSequentialAnimationGroup* sequentialAnimation = new QSequentialAnimationGroup( this );
    sequentialAnimation->addAnimation( greenBallAnimation );
    sequentialAnimation->addAnimation( redBallAnimation );
    d->ballAnimationGroup->addAnimation( sequentialAnimation );

    connect( animateButton, SIGNAL( clicked() ), SLOT( startAnimation() ) );
}

Harness::~Harness()
{
    delete d_ptr;
}

void Harness::startAnimation()
{
    Q_D( Harness );

    foreach ( QPropertyAnimation* propertyAnimation, findChildren<QPropertyAnimation*>() ) {
	MovableGraphicsPixmapItem* item = ( MovableGraphicsPixmapItem* )propertyAnimation->targetObject();
	item->setProperty( "pos", propertyAnimation->startValue() );
    }

    d->ballAnimationGroup->start();
}

MovableGraphicsPixmapItem* Harness::createBallItem( const QString& pixmapFileName )
{
    MovableGraphicsPixmapItem* item = new MovableGraphicsPixmapItem( QPixmap( pixmapFileName ) );

    return item;
}

QPropertyAnimation* Harness::createBallAnimation( MovableGraphicsPixmapItem* item, int duration, int startX, int endX, int y )
{
    QPropertyAnimation* ballAnimation = new QPropertyAnimation( item, "pos", this );
    ballAnimation->setDuration( duration );
    ballAnimation->setStartValue( QPoint( startX, y ) );
    ballAnimation->setEndValue( QPoint( endX, y ) );
    ballAnimation->setEasingCurve( QEasingCurve::OutBounce );
    
    return ballAnimation;
}

int main( int argc, char** argv )
{
    QApplication app( argc, argv );

    Harness harness;
    harness.show();

    return app.exec();
}

#include "movablegraphicsitem.moc"
